Exception: Minitest::ForkExecutor::UnmarshallableError

Inherits:
RuntimeError
  • Object
show all
Defined in:
lib/minitest/fork_executor.rb

Overview

An always marshallable exception class that can be derived from another (potentially non-marshallable exception). It's actually not intended to be raised but merely instantiated when passing Minitest failures from the runner to the reporter.

Instance Method Summary collapse

Constructor Details

#initialize(exc) ⇒ UnmarshallableError

Returns a new instance of UnmarshallableError.



163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
# File 'lib/minitest/fork_executor.rb', line 163

def initialize(exc)
  super(<<MESSAGE)
An unmarshallable error has occured. Below is its best-effort representation.
In order to receive the error itself, please disable Minitest::ForkExecutor.

Error class:
#{exc.class.name}

Error message:
#{exc.message}

Attributes:
#{exc.instance_variables.map do |name|
  "  #{name} = #{exc.instance_variable_get(name).inspect}"
end.join("\n")}

END OF ERROR MESSAGE (ORIGINAL BACKTRACE MAY FOLLOW)
MESSAGE

  set_backtrace(exc.backtrace)
end